@@ -8,6 +8,7 @@ SSL=${SSL:-nossl}
 URI=${URI:-}
 OS=$(uname -s | tr '[:upper:]' '[:lower:]')
 DNS=${DNS:-nodns}
+LOADBALANCED=${LOADBALANCED:-noloadbalanced}
 
 # AddressSanitizer configuration
 export ASAN_OPTIONS="detect_leaks=1 abort_on_error=1 symbolize=1"
@@ -49,12 +50,14 @@ if [ "$IPV4_ONLY" != "on" ]; then
    export MONGOC_CHECK_IPV6="on"
 fi
 
-if [ "$DNS" = "dns-auth" ]; then
-   export MONGOC_TEST_DNS=on
-   TEST_ARGS="$TEST_ARGS -l /initial_dns_auth/*"
-elif [ "$DNS" != "nodns" ]; then
-   export MONGOC_TEST_DNS=on
-   TEST_ARGS="$TEST_ARGS -l /initial_dns_seedlist_discovery*"
+# TODO (CDRIVER-4045): consolidate DNS tests into regular test tasks.
+if [ "$DNS" != "nodns" ]; then
+   TEST_ARGS="$TEST_ARGS -l /initial_dns_seedlist_discovery/*"
+   if [ "$DNS" = "loadbalanced" ]; then
+      export MONGOC_TEST_DNS_LOADBALANCED=on
+   else
+      export MONGOC_TEST_DNS=on
+   fi
 fi
 
 if [ "$CC" = "mingw" ]; then
@@ -89,6 +92,29 @@ check_mongocryptd() {
 
 export MONGOC_TEST_MONITORING_VERBOSE=on
 
+if [ "$LOADBALANCED" != "noloadbalanced" ]; then
+   if [ -z "$SINGLE_MONGOS_LB_URI" -o -z "$MULTI_MONGOS_LB_URI" ]; then
+      echo "SINGLE_MONGOS_LB_URI and MULTI_MONGOS_LB_URI environment variables required."
+      exit 1
+   fi
+   
+   export MONGOC_TEST_LOADBALANCED=ON
+
+   TEST_ARGS="$TEST_ARGS -l /unified/*"
+   TEST_ARGS="$TEST_ARGS -l /retryable_reads/*"
+   TEST_ARGS="$TEST_ARGS -l /retryable_writes/*"
+   TEST_ARGS="$TEST_ARGS -l /change_streams/*"
+   TEST_ARGS="$TEST_ARGS -l /loadbalanced/*"
+   TEST_ARGS="$TEST_ARGS -l /load_balancers/*"
+   TEST_ARGS="$TEST_ARGS -l /crud/unified/*"
+   TEST_ARGS="$TEST_ARGS -l /transactions/unified/*"
+   TEST_ARGS="$TEST_ARGS -l /collection-management/*"
+   TEST_ARGS="$TEST_ARGS -l /sessions/unified/*"
+   TEST_ARGS="$TEST_ARGS -l /change_streams/unified/*"
+   TEST_ARGS="$TEST_ARGS -l /versioned_api/*"
+   TEST_ARGS="$TEST_ARGS -l /command_monitoring/unified/*"
+fi
+
 case "$OS" in
    cygwin*)
       export PATH=$PATH:/cygdrive/c/mongodb/bin:/cygdrive/c/libmongocrypt/bin
