@@ -15,7 +15,7 @@
  */
 
 
-#include <bson.h>
+#include <bson/bson.h>
 
 #include "corpus-test.h"
 
@@ -84,12 +84,12 @@ corpus_test (bson_t *scenario,
 
    /* test valid BSON and Extended JSON */
    if (bson_iter_init_find (&iter, scenario, "valid")) {
-      bson_iter_recurse (&iter, &inner_iter);
+      BSON_ASSERT (bson_iter_recurse (&iter, &inner_iter));
       while (bson_iter_next (&inner_iter)) {
          bson_iter_t test_iter;
          test_bson_valid_type_t test = {scenario_description, bson_type, NULL};
 
-         bson_iter_recurse (&inner_iter, &test_iter);
+         BSON_ASSERT (bson_iter_recurse (&inner_iter, &test_iter));
          while (bson_iter_next (&test_iter)) {
             const char *key = bson_iter_key (&test_iter);
 
@@ -133,13 +133,13 @@ corpus_test (bson_t *scenario,
 
    /* test decoding errors (i.e. invalid BSON) */
    if (bson_iter_init_find (&iter, scenario, "decodeErrors")) {
-      bson_iter_recurse (&iter, &inner_iter);
+      BSON_ASSERT (bson_iter_recurse (&iter, &inner_iter));
       while (bson_iter_next (&inner_iter)) {
          bson_iter_t test_iter;
          test_bson_decode_error_type_t test = {
             scenario_description, bson_type, NULL};
 
-         bson_iter_recurse (&inner_iter, &test_iter);
+         BSON_ASSERT (bson_iter_recurse (&inner_iter, &test_iter));
          while (bson_iter_next (&test_iter)) {
             if (!strcmp (bson_iter_key (&test_iter), "description")) {
                test.test_description = bson_iter_utf8 (&test_iter, NULL);
@@ -160,13 +160,13 @@ corpus_test (bson_t *scenario,
 
    /* test parsing errors (e.g. invalid JSON or Decimal128 strings) */
    if (bson_iter_init_find (&iter, scenario, "parseErrors")) {
-      bson_iter_recurse (&iter, &inner_iter);
+      BSON_ASSERT (bson_iter_recurse (&iter, &inner_iter));
       while (bson_iter_next (&inner_iter)) {
          bson_iter_t test_iter;
          test_bson_parse_error_type_t test = {
             scenario_description, bson_type, NULL};
 
-         bson_iter_recurse (&inner_iter, &test_iter);
+         BSON_ASSERT (bson_iter_recurse (&inner_iter, &test_iter));
          while (bson_iter_next (&test_iter)) {
             if (!strcmp (bson_iter_key (&test_iter), "description")) {
                test.test_description = bson_iter_utf8 (&test_iter, NULL);
