@@ -7,6 +7,7 @@
 #include "mock_server/future-functions.h"
 #include "mongoc/mongoc-cursor-private.h"
 #include "mongoc/mongoc-collection-private.h"
+#include "mongoc/mongoc-error-private.h"
 #include "mongoc/mongoc-read-concern-private.h"
 #include "mongoc/mongoc-write-concern-private.h"
 #include "test-conveniences.h"
@@ -650,7 +651,12 @@ killcursors_succeeded (const mongoc_apm_command_succeeded_t *event)
 extern void
 _mongoc_cursor_impl_find_opquery_init (mongoc_cursor_t *cursor, bson_t *filter);
 
-/* test killing a cursor with mongo_cursor_destroy and a real server */
+/* Tests killing a cursor with mongo_cursor_destroy and a real server.
+ * Asserts that the cursor ID is no longer valid by attempting to get another
+ * batch of results with the previously killed cursor ID. Uses OP_GET_MORE (on
+ * servers older than 3.2) or a getMore command (servers 3.2+) to iterate the
+ * cursor ID.
+ */
 static void
 test_kill_cursor_live (void)
 {
@@ -702,22 +708,40 @@ test_kill_cursor_live (void)
 
    ASSERT_CMPINT (ctx.succeeded_count, ==, 1);
 
-   b = bson_new ();
-   cursor = _mongoc_cursor_find_new (
-      client, collection->ns, b, NULL, NULL, NULL, NULL);
-   /* override the typical priming, and immediately transition to an OPQUERY
-    * find cursor. */
-   cursor->impl.destroy (&cursor->impl);
-   _mongoc_cursor_impl_find_opquery_init (cursor, b);
+   if (test_framework_supports_legacy_opcodes ()) {
+      b = bson_new ();
+      cursor = _mongoc_cursor_find_new (
+         client, collection->ns, b, NULL, NULL, NULL, NULL);
+      /* override the typical priming, and immediately transition to an OPQUERY
+      * find cursor. */
+      cursor->impl.destroy (&cursor->impl);
+      _mongoc_cursor_impl_find_opquery_init (cursor, b);
 
-   cursor->cursor_id = ctx.cursor_id;
-   cursor->state = END_OF_BATCH; /* meaning, "finished reading first batch" */
-   r = mongoc_cursor_next (cursor, &doc);
-   ASSERT (!r);
-   ASSERT (mongoc_cursor_error (cursor, &error));
-   ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_CURSOR, 16, "cursor is invalid");
+      cursor->cursor_id = ctx.cursor_id;
+      cursor->state = END_OF_BATCH; /* meaning, "finished reading first batch" */
+      r = mongoc_cursor_next (cursor, &doc);
+      ASSERT (!r);
+      ASSERT (mongoc_cursor_error (cursor, &error));
+      ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_CURSOR, 16, "cursor is invalid");
+
+      mongoc_cursor_destroy (cursor);
+   } else {
+      bson_t *cmd;
+
+      cmd = BCON_NEW ("getMore",
+                      BCON_INT64 (ctx.cursor_id),
+                      "collection",
+                      mongoc_collection_get_name (collection));
+      r = mongoc_client_command_simple (
+         client, "test", cmd, NULL /* read prefs */, NULL /* reply */, &error);
+      ASSERT (!r);
+      ASSERT_ERROR_CONTAINS (error,
+                             MONGOC_ERROR_QUERY,
+                             MONGOC_SERVER_ERR_CURSOR_NOT_FOUND,
+                             "not found");
+      bson_destroy (cmd);
+   }
 
-   mongoc_cursor_destroy (cursor);
    mongoc_bulk_operation_destroy (bulk);
    mongoc_collection_destroy (collection);
    mongoc_client_destroy (client);
