@@ -42,7 +42,7 @@ get_generation (mongoc_client_t *client, mongoc_cursor_t *cursor)
    sd = mongoc_topology_description_server_by_id (
       &client->topology->description, server_id, &error);
    ASSERT_OR_PRINT (sd, error);
-   generation = sd->generation;
+   generation = mongoc_generation_map_get (sd->generation_map, &kZeroServiceId);
    bson_mutex_unlock (&client->topology->mutex);
 
    return generation;
@@ -666,19 +666,22 @@ test_exhaust_install (TestSuite *suite)
                       test_exhaust_cursor_single,
                       NULL,
                       NULL,
-                      skip_if_mongos);
+                      skip_if_mongos,
+                      test_framework_skip_if_no_legacy_opcodes);
    TestSuite_AddFull (suite,
                       "/Client/exhaust_cursor/pool",
                       test_exhaust_cursor_pool,
                       NULL,
                       NULL,
-                      skip_if_mongos);
+                      skip_if_mongos,
+                      test_framework_skip_if_no_legacy_opcodes);
    TestSuite_AddFull (suite,
                       "/Client/exhaust_cursor/batches",
                       test_exhaust_cursor_multi_batch,
                       NULL,
                       NULL,
-                      skip_if_mongos);
+                      skip_if_mongos,
+                      test_framework_skip_if_no_legacy_opcodes);
    TestSuite_AddLive (suite,
                       "/Client/set_max_await_time_ms",
                       test_cursor_set_max_await_time_ms);
