@@ -21,6 +21,7 @@
 #include "TestSuite.h"
 #include "mock_server/mock-server.h"
 #include "mock_server/future-functions.h"
+#include "test-libmongoc.h"
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "client-test-hedged-reads"
@@ -40,7 +41,8 @@ test_mongos_hedged_reads_read_pref (void)
 
    server = mock_mongos_new (5);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
 
    prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY_PREFERRED);
@@ -51,12 +53,12 @@ test_mongos_hedged_reads_read_pref (void)
 
    future = future_collection_count (
       collection, MONGOC_QUERY_NONE, NULL, 0, 0, NULL, &error);
-   request = mock_server_receives_command (
-      server,
-      "db",
-      MONGOC_QUERY_SLAVE_OK,
-      "{'$readPreference': { '$exists': false }}",
-      NULL);
+   request =
+      mock_server_receives_command (server,
+                                    "db",
+                                    MONGOC_QUERY_SECONDARY_OK,
+                                    "{'$readPreference': { '$exists': false }}",
+                                    NULL);
 
    mock_server_replies_simple (request, "{'ok': 1, 'n': 1}");
    ASSERT_OR_PRINT (1 == future_get_int64_t (future), error);
@@ -76,7 +78,7 @@ test_mongos_hedged_reads_read_pref (void)
    request = mock_server_receives_command (
       server,
       "db",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'$readPreference': "
       " {'mode': 'secondaryPreferred', 'hedge': {'enabled': true}}}",
       NULL);
@@ -98,7 +100,6 @@ test_mongos_hedged_reads_read_pref (void)
 void
 test_client_hedged_reads_install (TestSuite *suite)
 {
-   TestSuite_AddMockServerTest (suite,
-                                "/Client/hedged_reads/mongos",
-                                test_mongos_hedged_reads_read_pref);
+   TestSuite_AddMockServerTest (
+      suite, "/Client/hedged_reads/mongos", test_mongos_hedged_reads_read_pref);
 }
