@@ -7,7 +7,7 @@
 #include "mock_server/mock-server.h"
 #include "mock_server/mock-rs.h"
 #include "test-conveniences.h"
-
+#include "test-libmongoc.h"
 
 static bool
 _can_be_command (const char *query)
@@ -31,7 +31,7 @@ _test_op_query (const mongoc_uri_t *uri,
    future_t *future;
    request_t *request;
 
-   client = mongoc_client_new_from_uri (uri);
+   client = test_framework_client_new_from_uri (uri, NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
 
    cursor = mongoc_collection_find (collection,
@@ -83,7 +83,7 @@ _test_find_command (const mongoc_uri_t *uri,
    future_t *future;
    request_t *request;
 
-   client = mongoc_client_new_from_uri (uri);
+   client = test_framework_client_new_from_uri (uri, NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
 
    cursor = mongoc_collection_find (collection,
@@ -138,7 +138,7 @@ _test_op_msg (const mongoc_uri_t *uri,
    future_t *future;
    request_t *request;
 
-   client = mongoc_client_new_from_uri (uri);
+   client = test_framework_client_new_from_uri (uri, NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
 
    cursor = mongoc_collection_find (collection,
@@ -187,7 +187,7 @@ _test_command (const mongoc_uri_t *uri,
    future_t *future;
    request_t *request;
 
-   client = mongoc_client_new_from_uri (uri);
+   client = test_framework_client_new_from_uri (uri, NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
    mongoc_collection_set_read_prefs (collection, read_prefs);
 
@@ -236,7 +236,7 @@ _test_command_simple (const mongoc_uri_t *uri,
    future_t *future;
    request_t *request;
 
-   client = mongoc_client_new_from_uri (uri);
+   client = test_framework_client_new_from_uri (uri, NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
    mongoc_collection_set_read_prefs (collection, read_prefs);
 
@@ -282,40 +282,40 @@ _run_server (read_pref_test_type_t test_type, int32_t max_wire_version)
    BSON_ASSERT (max_wire_version > 0);
    switch (test_type) {
    case READ_PREF_TEST_STANDALONE:
-      mock_server_auto_ismaster (server,
-                                 "{'ok': 1,"
-                                 " 'maxWireVersion': %d,"
-                                 " 'ismaster': true}",
-                                 max_wire_version);
+      mock_server_auto_hello (server,
+                              "{'ok': 1,"
+                              " 'maxWireVersion': %d,"
+                              " 'isWritablePrimary': true}",
+                              max_wire_version);
       break;
    case READ_PREF_TEST_MONGOS:
-      mock_server_auto_ismaster (server,
-                                 "{'ok': 1,"
-                                 " 'maxWireVersion': %d,"
-                                 " 'ismaster': true,"
-                                 " 'msg': 'isdbgrid'}",
-                                 max_wire_version);
+      mock_server_auto_hello (server,
+                              "{'ok': 1,"
+                              " 'maxWireVersion': %d,"
+                              " 'isWritablePrimary': true,"
+                              " 'msg': 'isdbgrid'}",
+                              max_wire_version);
       break;
    case READ_PREF_TEST_PRIMARY:
-      mock_server_auto_ismaster (server,
-                                 "{'ok': 1,"
-                                 " 'maxWireVersion': %d,"
-                                 " 'ismaster': true,"
-                                 " 'setName': 'rs',"
-                                 " 'hosts': ['%s']}",
-                                 max_wire_version,
-                                 mock_server_get_host_and_port (server));
+      mock_server_auto_hello (server,
+                              "{'ok': 1,"
+                              " 'maxWireVersion': %d,"
+                              " 'isWritablePrimary': true,"
+                              " 'setName': 'rs',"
+                              " 'hosts': ['%s']}",
+                              max_wire_version,
+                              mock_server_get_host_and_port (server));
       break;
    case READ_PREF_TEST_SECONDARY:
-      mock_server_auto_ismaster (server,
-                                 "{'ok': 1,"
-                                 " 'maxWireVersion': %d,"
-                                 " 'ismaster': false,"
-                                 " 'secondary': true,"
-                                 " 'setName': 'rs',"
-                                 " 'hosts': ['%s']}",
-                                 max_wire_version,
-                                 mock_server_get_host_and_port (server));
+      mock_server_auto_hello (server,
+                              "{'ok': 1,"
+                              " 'maxWireVersion': %d,"
+                              " 'isWritablePrimary': false,"
+                              " 'secondary': true,"
+                              " 'setName': 'rs',"
+                              " 'hosts': ['%s']}",
+                              max_wire_version,
+                              mock_server_get_host_and_port (server));
       break;
    default:
       fprintf (stderr, "Invalid test_type: : %d\n", test_type);
@@ -437,9 +437,9 @@ test_read_prefs_standalone_null (void)
                             NULL,
                             "{}",
                             "{}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter': {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {}, "
                             "'$readPreference': { '$exists': false } }");
 
@@ -447,9 +447,9 @@ test_read_prefs_standalone_null (void)
                             NULL,
                             "{'a': 1}",
                             "{'a': 1}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter': {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {'a': 1}, "
                             "'$readPreference': { '$exists': false } }");
 }
@@ -460,7 +460,7 @@ test_read_prefs_standalone_primary (void)
    mongoc_read_prefs_t *read_prefs;
 
    /* Server Selection Spec: for topology type single and server types other
-    * than mongos, "clients MUST always set the slaveOK wire protocol flag on
+    * than mongos, "clients MUST always set the secondaryOk wire protocol flag on
     * reads to ensure that any server type can handle the request."
     * */
    read_prefs = mongoc_read_prefs_new (MONGOC_READ_PRIMARY);
@@ -469,9 +469,9 @@ test_read_prefs_standalone_primary (void)
                             read_prefs,
                             "{}",
                             "{}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter':  {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {}, "
                             "'$readPreference': { '$exists': false } }");
 
@@ -479,9 +479,9 @@ test_read_prefs_standalone_primary (void)
                             read_prefs,
                             "{'a': 1}",
                             "{'a': 1}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter':  {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {'a': 1}, "
                             "'$readPreference': { '$exists': false } }");
 
@@ -500,9 +500,9 @@ test_read_prefs_standalone_secondary (void)
                             read_prefs,
                             "{}",
                             "{}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter':  {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {}, "
                             "'$readPreference': { '$exists': false } }");
 
@@ -510,9 +510,9 @@ test_read_prefs_standalone_secondary (void)
                             read_prefs,
                             "{'a': 1}",
                             "{'a': 1}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter':  {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {'a': 1}, "
                             "'$readPreference': { '$exists': false } }");
 
@@ -536,9 +536,9 @@ test_read_prefs_standalone_tags (void)
                             read_prefs,
                             "{}",
                             "{}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter':  {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {}, "
                             "'$readPreference': { '$exists': false } }");
 
@@ -546,9 +546,9 @@ test_read_prefs_standalone_tags (void)
                             read_prefs,
                             "{'a': 1}",
                             "{'a': 1}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{'find': 'test', 'filter':  {}}",
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             "{ 'find': 'test', 'filter': {'a': 1}, "
                             "'$readPreference': { '$exists': false } }");
 
@@ -595,17 +595,17 @@ test_read_prefs_secondary_rssecondary (void)
                      read_prefs,
                      "{}",
                      "{}",
-                     MONGOC_QUERY_SLAVE_OK,
+                     MONGOC_QUERY_SECONDARY_OK,
                      "{'find': 'test', 'filter':  {}}",
-                     MONGOC_QUERY_SLAVE_OK);
+                     MONGOC_QUERY_SECONDARY_OK);
 
    _test_read_prefs (READ_PREF_TEST_SECONDARY,
                      read_prefs,
                      "{'a': 1}",
                      "{'a': 1}",
-                     MONGOC_QUERY_SLAVE_OK,
+                     MONGOC_QUERY_SECONDARY_OK,
                      "{'find': 'test', 'filter':  {'a': 1}}",
-                     MONGOC_QUERY_SLAVE_OK);
+                     MONGOC_QUERY_SECONDARY_OK);
 
    mongoc_read_prefs_destroy (read_prefs);
 }
@@ -672,10 +672,10 @@ test_read_prefs_mongos_secondary (void)
       read_prefs,
       "{}",
       "{'$query': {}, '$readPreference': {'mode': 'secondary'}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'$query': {'find': 'test', 'filter':  {}},"
       " '$readPreference': {'mode': 'secondary'}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'find': 'test', 'filter':  {},"
       " '$readPreference': {'mode': 'secondary'}}");
 
@@ -684,10 +684,10 @@ test_read_prefs_mongos_secondary (void)
       read_prefs,
       "{'a': 1}",
       "{'$query': {'a': 1}, '$readPreference': {'mode': 'secondary'}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'$query': {'find': 'test', 'filter':  {'a': 1}},"
       " '$readPreference': {'mode': 'secondary'}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'find': 'test', 'filter':  {'a': 1},"
       " '$readPreference': {'mode': 'secondary'}}");
 
@@ -696,10 +696,10 @@ test_read_prefs_mongos_secondary (void)
       read_prefs,
       "{'$query': {'a': 1}}",
       "{'$query': {'a': 1}, '$readPreference': {'mode': 'secondary'}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'$query': {'find': 'test', 'filter':  {'a': 1}},"
       " '$readPreference': {'mode': 'secondary'}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'find': 'test', 'filter':  {'a': 1},"
       " '$readPreference': {'mode': 'secondary'}}");
 
@@ -714,22 +714,22 @@ test_read_prefs_mongos_secondary_preferred (void)
 
    read_prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY_PREFERRED);
 
-   /* $readPreference not sent, only slaveOk */
+   /* $readPreference not sent, only secondaryOk */
    _test_read_prefs (READ_PREF_TEST_MONGOS,
                      read_prefs,
                      "{}",
                      "{}",
-                     MONGOC_QUERY_SLAVE_OK,
+                     MONGOC_QUERY_SECONDARY_OK,
                      "{'find': 'test', 'filter':  {}}",
-                     MONGOC_QUERY_SLAVE_OK);
+                     MONGOC_QUERY_SECONDARY_OK);
 
    _test_read_prefs (READ_PREF_TEST_MONGOS,
                      read_prefs,
                      "{'a': 1}",
                      "{'a': 1}",
-                     MONGOC_QUERY_SLAVE_OK,
+                     MONGOC_QUERY_SECONDARY_OK,
                      "{'find': 'test', 'filter':  {'a': 1}}",
-                     MONGOC_QUERY_SLAVE_OK);
+                     MONGOC_QUERY_SECONDARY_OK);
 
    mongoc_read_prefs_destroy (read_prefs);
 }
@@ -753,11 +753,11 @@ test_read_prefs_mongos_tags (void)
       "{}",
       "{'$query': {}, '$readPreference': {'mode': 'secondaryPreferred',"
       "                                   'tags': [{'dc': 'ny'}, {}]}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'$query': {'find': 'test', 'filter':  {}},"
       " '$readPreference': {'mode': 'secondaryPreferred',"
       "                             'tags': [{'dc': 'ny'}, {}]}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'find': 'test', 'filter':  {},"
       " '$readPreference': {'mode': 'secondaryPreferred',"
       "                             'tags': [{'dc': 'ny'}, {}]}}");
@@ -769,11 +769,11 @@ test_read_prefs_mongos_tags (void)
       "{'$query': {'a': 1},"
       " '$readPreference': {'mode': 'secondaryPreferred',"
       "                     'tags': [{'dc': 'ny'}, {}]}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'$query': {'find': 'test', 'filter':  {}},"
       " '$readPreference': {'mode': 'secondaryPreferred',"
       "                             'tags': [{'dc': 'ny'}, {}]}}",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'find': 'test', 'filter':  {},"
       " '$readPreference': {'mode': 'secondaryPreferred',"
       "                             'tags': [{'dc': 'ny'}, {}]}}");
@@ -798,7 +798,8 @@ test_read_prefs_mongos_max_staleness (void)
 
    server = mock_mongos_new (WIRE_VERSION_MAX_STALENESS);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
 
    prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY_PREFERRED);
@@ -812,7 +813,7 @@ test_read_prefs_mongos_max_staleness (void)
    request = mock_server_receives_query (
       server,
       "test.test",
-      MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SECONDARY_OK,
       0,
       0,
       "{'$query': {'a': 1},"
@@ -821,7 +822,7 @@ test_read_prefs_mongos_max_staleness (void)
       "{}");
 
    mock_server_replies_to_find (request,
-                                MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SLAVE_OK,
+                                MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SECONDARY_OK,
                                 0,
                                 1,
                                 "test.test",
@@ -856,7 +857,8 @@ test_read_prefs_mongos_hedged_reads (void)
 
    server = mock_mongos_new (WIRE_VERSION_HEDGED_READS);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
 
    prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY_PREFERRED);
@@ -872,7 +874,7 @@ test_read_prefs_mongos_hedged_reads (void)
    request = mock_server_receives_query (
       server,
       "test.test",
-      MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SECONDARY_OK,
       0,
       0,
       "{'$query': {'a': 1},"
@@ -881,7 +883,7 @@ test_read_prefs_mongos_hedged_reads (void)
       "{}");
 
    mock_server_replies_to_find (request,
-                                MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SLAVE_OK,
+                                MONGOC_QUERY_EXHAUST | MONGOC_QUERY_SECONDARY_OK,
                                 0,
                                 1,
                                 "test.test",
@@ -916,7 +918,8 @@ test_mongos_read_concern (void)
 
    server = mock_mongos_new (WIRE_VERSION_READ_CONCERN);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
    prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
    cursor = mongoc_collection_find_with_opts (
@@ -929,7 +932,7 @@ test_mongos_read_concern (void)
    request = mock_server_receives_command (
       server,
       "test",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{"
       "  '$query': {"
       "    'find': 'test', 'filter': {}, 'readConcern': {'level': 'foo'}"
@@ -941,7 +944,7 @@ test_mongos_read_concern (void)
       "}");
 
    mock_server_replies_to_find (
-      request, MONGOC_QUERY_SLAVE_OK, 0, 1, "db.collection", "{}", true);
+      request, MONGOC_QUERY_SECONDARY_OK, 0, 1, "db.collection", "{}", true);
 
    /* mongoc_cursor_next returned true */
    BSON_ASSERT (future_get_bool (future));
@@ -961,7 +964,7 @@ typedef mongoc_cursor_t *(*test_op_msg_direct_fn_t) (mongoc_collection_t *,
 
 
 /* direct connection to a secondary requires read pref primaryPreferred to
- * avoid "not master" error from server */
+ * avoid "not primary" error from server */
 static void
 _test_op_msg_direct_connection (bool is_mongos,
                                 test_op_msg_direct_fn_t fn,
@@ -982,22 +985,23 @@ _test_op_msg_direct_connection (bool is_mongos,
    if (is_mongos) {
       server = mock_mongos_new (WIRE_VERSION_OP_MSG);
    } else {
-      char* ismaster = bson_strdup_printf ("{'ok': 1.0,"
-                                        " 'ismaster': true,"
+      char *hello = bson_strdup_printf ("{'ok': 1.0,"
+                                        " 'isWritablePrimary': true,"
                                         " 'setName': 'rs0',"
                                         " 'secondary': true,"
                                         " 'minWireVersion': 0,"
                                         " 'maxWireVersion': %d}",
                                         WIRE_VERSION_OP_MSG);
       server = mock_server_new ();
-      mock_server_auto_ismaster (server, ismaster);
-      bson_free (ismaster);
+      mock_server_auto_hello (server, hello);
+      bson_free (hello);
    }
 
    mock_server_auto_endsessions (server);
 
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
 
    for (i = 0; i < 2; i++) {
@@ -1047,7 +1051,7 @@ aggregate (mongoc_collection_t *collection, mongoc_read_prefs_t *prefs)
 
 
 /* direct connection to a secondary requires read pref primaryPreferred to
- * avoid "not master" error from server */
+ * avoid "not primary" error from server */
 static void
 test_op_msg_direct_secondary ()
 {
