@@ -107,9 +107,9 @@ _test_topology_reconcile_rs (bool pooled)
    mock_server_run (server1);
 
    /* secondary, no tags */
-   RS_RESPONSE_TO_ISMASTER (server0, 6, false, false, server0, server1);
+   RS_RESPONSE_TO_HELLO (server0, 6, false, false, server0, server1);
    /* primary, no tags */
-   RS_RESPONSE_TO_ISMASTER (server1, 6, true, false, server0, server1);
+   RS_RESPONSE_TO_HELLO (server1, 6, true, false, server0, server1);
 
    /* provide secondary in seed list */
    uri_str = bson_strdup_printf ("mongodb://%s/?replicaSet=rs",
@@ -118,10 +118,10 @@ _test_topology_reconcile_rs (bool pooled)
    uri = mongoc_uri_new (uri_str);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new (uri_str);
+      client = test_framework_client_new (uri_str, NULL);
    }
 
    if (!pooled) {
@@ -150,7 +150,7 @@ _test_topology_reconcile_rs (bool pooled)
    /*
     * remove server1 from set. server0 is the primary, with tags.
     */
-   RS_RESPONSE_TO_ISMASTER (
+   RS_RESPONSE_TO_HELLO (
       server0, 6, true, true, server0); /* server1 absent */
 
    BSON_ASSERT (selects_server (client, tag_read_prefs, server0));
@@ -165,8 +165,8 @@ _test_topology_reconcile_rs (bool pooled)
    /*
     * server1 returns as a secondary. its scanner node is un-retired.
     */
-   RS_RESPONSE_TO_ISMASTER (server0, 6, true, true, server0, server1);
-   RS_RESPONSE_TO_ISMASTER (server1, 6, false, false, server0, server1);
+   RS_RESPONSE_TO_HELLO (server0, 6, true, true, server0, server1);
+   RS_RESPONSE_TO_HELLO (server1, 6, false, false, server0, server1);
 
    BSON_ASSERT (selects_server (client, secondary_read_prefs, server1));
 
@@ -236,10 +236,10 @@ _test_topology_reconcile_sharded (bool pooled)
    uri = mongoc_uri_new (uri_str);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new (uri_str);
+      client = test_framework_client_new (uri_str, NULL);
    }
 
    primary_read_prefs = mongoc_read_prefs_new (MONGOC_READ_PRIMARY);
@@ -247,10 +247,11 @@ _test_topology_reconcile_sharded (bool pooled)
       client->topology, MONGOC_SS_READ, primary_read_prefs, &error);
 
    /* mongos */
-   request = mock_server_receives_ismaster (mongos);
-   mock_server_replies_simple (request,
-                               "{'ok': 1, 'ismaster': true, 'minWireVersion': "
-                               "2, 'maxWireVersion': 5, 'msg': 'isdbgrid'}");
+   request = mock_server_receives_legacy_hello (mongos, NULL);
+   mock_server_replies_simple (
+      request,
+      "{'ok': 1, 'isWritablePrimary': true, 'minWireVersion': "
+      "2, 'maxWireVersion': 5, 'msg': 'isdbgrid'}");
 
    request_destroy (request);
 
@@ -258,11 +259,11 @@ _test_topology_reconcile_sharded (bool pooled)
    _mongoc_usleep (1000 * 1000);
 
    /* replica set secondary - topology removes it */
-   request = mock_server_receives_ismaster (secondary);
+   request = mock_server_receives_legacy_hello (secondary, NULL);
    secondary_response =
       bson_strdup_printf ("{'ok': 1, "
                           " 'setName': 'rs',"
-                          " 'ismaster': false,"
+                          " 'isWritablePrimary': false,"
                           " 'secondary': true,"
                           " 'minWireVersion': 2,"
                           " 'maxWireVersion': 5,"
@@ -367,7 +368,7 @@ test_topology_reconcile_from_handshake (void *ctx)
       "mongodb://%s/?replicaSet=%s", host_and_port, replset_name);
 
    uri = mongoc_uri_new (uri_str);
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
    mongoc_apm_set_server_opening_cb (callbacks, server_opening);
    mongoc_client_pool_set_apm_callbacks (pool, callbacks, &data);
    test_framework_set_pool_ssl_opts (pool);
@@ -380,14 +381,15 @@ test_topology_reconcile_from_handshake (void *ctx)
    /* ordinarily would start bg thread */
    client = mongoc_client_pool_pop (pool);
 
-   /* command in the foreground (ismaster, just because it doesn't need auth) */
-   r = mongoc_client_read_command_with_opts (client,
-                                             "admin",
-                                             tmp_bson ("{'ismaster': 1}"),
-                                             NULL,
-                                             tmp_bson ("{'serverId': 1}"),
-                                             NULL,
-                                             &error);
+   /* command in the foreground (hello, just because it doesn't need auth) */
+   r = mongoc_client_read_command_with_opts (
+      client,
+      "admin",
+      tmp_bson ("{'" HANDSHAKE_CMD_LEGACY_HELLO "': 1}"),
+      NULL,
+      tmp_bson ("{'serverId': 1}"),
+      NULL,
+      &error);
 
    ASSERT_OR_PRINT (r, error);
 
@@ -411,7 +413,13 @@ test_topology_reconcile_from_handshake (void *ctx)
 
    /* no serverId, waits for topology scan */
    r = mongoc_client_read_command_with_opts (
-      client, "admin", tmp_bson ("{'ismaster': 1}"), NULL, NULL, NULL, &error);
+      client,
+      "admin",
+      tmp_bson ("{'" HANDSHAKE_CMD_LEGACY_HELLO "': 1}"),
+      NULL,
+      NULL,
+      NULL,
+      &error);
 
    ASSERT_OR_PRINT (r, error);
    bson_mutex_lock (&data.mutex);
@@ -433,7 +441,7 @@ test_topology_reconcile_from_handshake (void *ctx)
  *
  * 1. scanner discovers a replica set with primary and at least one secondary
  * 2. cluster opens a new stream to the primary
- * 3. cluster handshakes the new connection by calling isMaster on the primary
+ * 3. cluster handshakes the new connection by calling hello on the primary
  * 4. the primary, for some reason, suddenly omits the secondary from its host
  *    list, perhaps because the secondary was removed from the RS configuration
  * 5. scanner marks the secondary scanner node "retired" to be destroyed later
@@ -468,8 +476,8 @@ test_topology_reconcile_retire_single (void)
    mock_server_run (secondary);
    mock_server_run (primary);
 
-   RS_RESPONSE_TO_ISMASTER (primary, 6, true, false, secondary, primary);
-   RS_RESPONSE_TO_ISMASTER (secondary, 6, false, false, secondary, primary);
+   RS_RESPONSE_TO_HELLO (primary, 6, true, false, secondary, primary);
+   RS_RESPONSE_TO_HELLO (secondary, 6, false, false, secondary, primary);
 
    /* selection timeout must be > MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS,
     * otherwise we skip second scan in pooled mode and don't hit the assert */
@@ -481,7 +489,7 @@ test_topology_reconcile_retire_single (void)
 
    uri = mongoc_uri_new (uri_str);
 
-   client = mongoc_client_new (uri_str);
+   client = test_framework_client_new (uri_str, NULL);
    topology = client->topology;
 
 
@@ -493,7 +501,7 @@ test_topology_reconcile_retire_single (void)
 
    /* remove secondary from primary's config */
    bson_mutex_lock (&topology->mutex);
-   RS_RESPONSE_TO_ISMASTER (primary, 6, true, false, primary);
+   RS_RESPONSE_TO_HELLO (primary, 6, true, false, primary);
 
    /* step 2: cluster opens new stream to primary - force new stream in single
     * mode by disconnecting scanner nodes (also includes step 6) */
@@ -557,7 +565,7 @@ test_topology_reconcile_retire_single (void)
  *
  * 1. scanner discovers a replica set with primary
  * 2. cluster opens a new stream to the primary
- * 3. cluster handshakes the new connection by calling isMaster on the primary
+ * 3. cluster handshakes the new connection by calling hello on the primary
  * 4. the primary suddenly includes a new secondary in its host list, perhaps
  *    because the secondary was added
  * 5. _mongoc_topology_update_from_handshake adds the secondary to the topology
@@ -587,9 +595,9 @@ test_topology_reconcile_add_single (void)
    mock_server_run (secondary);
    mock_server_run (primary);
 
-   /* omit secondary from primary's ismaster, to start with */
-   RS_RESPONSE_TO_ISMASTER (primary, 6, true, false, primary);
-   RS_RESPONSE_TO_ISMASTER (secondary, 6, false, false, secondary, primary);
+   /* omit secondary from primary's hello, to start with */
+   RS_RESPONSE_TO_HELLO (primary, 6, true, false, primary);
+   RS_RESPONSE_TO_HELLO (secondary, 6, false, false, secondary, primary);
 
    /* selection timeout must be > MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS,
     * otherwise we skip second scan in pooled mode and don't hit the assert */
@@ -601,7 +609,7 @@ test_topology_reconcile_add_single (void)
 
    uri = mongoc_uri_new (uri_str);
 
-   client = mongoc_client_new (uri_str);
+   client = test_framework_client_new (uri_str, NULL);
    topology = client->topology;
 
    /* step 1: discover primary */
@@ -609,7 +617,7 @@ test_topology_reconcile_add_single (void)
    BSON_ASSERT (selects_server (client, primary_read_prefs, primary));
 
    /* add secondary to primary's config */
-   RS_RESPONSE_TO_ISMASTER (primary, 6, true, false, primary, secondary);
+   RS_RESPONSE_TO_HELLO (primary, 6, true, false, primary, secondary);
 
    /* step 2: cluster opens new stream to primary - force new stream in single
     * mode by disconnecting primary scanner node */
