@@ -31,6 +31,11 @@
 #define REDUCED_HEARTBEAT_FREQUENCY_MS 500
 #define REDUCED_MIN_HEARTBEAT_FREQUENCY_MS 50
 
+struct _entity_findcursor_t {
+   const bson_t *first_result;
+   mongoc_cursor_t *cursor;
+};
+
 static void
 entity_destroy (entity_t *entity);
 
@@ -85,6 +90,8 @@ uri_apply_options (mongoc_uri_t *uri, bson_t *opts, bson_error_t *error)
          mongoc_uri_set_option_as_int64 (uri, key, bson_iter_int64 (&iter));
       } else if (mongoc_uri_option_is_bool (key)) {
          mongoc_uri_set_option_as_bool (uri, key, bson_iter_bool (&iter));
+      } else if (0 == strcmp ("appname", key)) {
+         mongoc_uri_set_appname (uri, bson_iter_utf8 (&iter, NULL));
       } else {
          test_set_error (
             error, "Unimplemented test runner support for URI option: %s", key);
@@ -197,6 +204,11 @@ command_started (const mongoc_apm_command_started_t *started)
    event->database_name =
       bson_strdup (mongoc_apm_command_started_get_database_name (started));
 
+   if (mongoc_apm_command_started_get_service_id (started)) {
+      bson_oid_copy (mongoc_apm_command_started_get_service_id (started),
+                     &event->service_id);
+   }
+
    if (should_ignore_event (entity, event)) {
       event_destroy (event);
       return;
@@ -217,6 +229,11 @@ command_failed (const mongoc_apm_command_failed_t *failed)
    event->command_name =
       bson_strdup (mongoc_apm_command_failed_get_command_name (failed));
 
+   if (mongoc_apm_command_failed_get_service_id (failed)) {
+      bson_oid_copy (mongoc_apm_command_failed_get_service_id (failed),
+                     &event->service_id);
+   }
+
    if (should_ignore_event (entity, event)) {
       event_destroy (event);
       return;
@@ -237,6 +254,11 @@ command_succeeded (const mongoc_apm_command_succeeded_t *succeeded)
    event->command_name =
       bson_strdup (mongoc_apm_command_succeeded_get_command_name (succeeded));
 
+   if (mongoc_apm_command_succeeded_get_service_id (succeeded)) {
+      bson_oid_copy (mongoc_apm_command_succeeded_get_service_id (succeeded),
+                     &event->service_id);
+   }
+
    if (should_ignore_event (entity, event)) {
       event_destroy (event);
       return;
@@ -257,6 +279,7 @@ entity_client_new (entity_map_t *em, bson_t *bson, bson_error_t *error)
    bson_t *uri_options = NULL;
    bool *use_multiple_mongoses = NULL;
    bson_t *observe_events = NULL;
+   bson_t *store_events_as_entities = NULL;
    bson_t *server_api = NULL;
    bool can_reduce_heartbeat = false;
    mongoc_server_api_t *api = NULL;
@@ -274,6 +297,8 @@ entity_client_new (entity_map_t *em, bson_t *bson, bson_error_t *error)
    bson_parser_doc_optional (parser, "serverApi", &server_api);
    bson_parser_bool_optional (
       parser, "observeSensitiveCommands", &entity->observe_sensitive_commands);
+   bson_parser_array_optional (
+      parser, "storeEventsAsEntities", &store_events_as_entities);
 
    if (!bson_parser_parse (parser, bson, error)) {
       goto done;
@@ -318,9 +343,20 @@ entity_client_new (entity_map_t *em, bson_t *bson, bson_error_t *error)
     * host. If useMultipleMongoses is false, require that the connection string
     * has one host. If useMultipleMongoses unspecified, make no assertion.
     */
-   if (use_multiple_mongoses != NULL) {
+   if (test_framework_is_loadbalanced ()) {
+      /* Quoting the unified test runner specification:
+       * If the topology type is LoadBalanced, [...] If useMultipleMongoses is
+       * true or unset, the test runner MUST use the URI of the load balancer
+       * fronting multiple servers. Otherwise, the test runner MUST use the URI
+       * of the load balancer fronting a single server.
+       */
+      if (use_multiple_mongoses == NULL || *use_multiple_mongoses == true) {
+         mongoc_uri_destroy (uri);
+         uri = test_framework_get_uri_multi_mongos_loadbalanced ();
+      }
+   } else if (use_multiple_mongoses != NULL) {
       if (!test_framework_uri_apply_multi_mongos (
-             uri, *use_multiple_mongoses, error)) {
+               uri, *use_multiple_mongoses, error)) {
          goto done;
       }
    }
@@ -362,6 +398,9 @@ entity_client_new (entity_map_t *em, bson_t *bson, bson_error_t *error)
             mongoc_apm_set_command_failed_cb (callbacks, command_failed);
          } else if (0 == strcmp (event_type, "commandSucceededEvent")) {
             mongoc_apm_set_command_succeeded_cb (callbacks, command_succeeded);
+         } else if (is_unsupported_event_type (event_type)) {
+            MONGOC_DEBUG ("Skipping observing unsupported event type: %s", event_type);
+            continue;
          } else {
             test_set_error (error, "Unexpected event type: %s", event_type);
             goto done;
@@ -370,6 +409,12 @@ entity_client_new (entity_map_t *em, bson_t *bson, bson_error_t *error)
    }
    mongoc_client_set_apm_callbacks (client, callbacks, entity);
 
+   if (store_events_as_entities) {
+      /* TODO: CDRIVER-3867 Comprehensive Atlas Testing */
+      test_set_error (error, "storeEventsAsEntities is not supported");
+      goto done;
+   }
+
    ret = true;
 done:
    mongoc_uri_destroy (uri);
@@ -379,6 +424,7 @@ done:
    bson_destroy (uri_options);
    bson_free (use_multiple_mongoses);
    bson_destroy (observe_events);
+   bson_destroy (store_events_as_entities);
    bson_destroy (server_api);
    if (!ret) {
       entity_destroy (entity);
@@ -566,6 +612,7 @@ session_opts_new (bson_t *bson, bson_error_t *error)
    bson_parser_t *bp = NULL;
    bson_parser_t *bp_opts = NULL;
    bool *causal_consistency = NULL;
+   bool *snapshot = NULL;
    bson_t *default_transaction_opts = NULL;
    mongoc_write_concern_t *wc = NULL;
    mongoc_read_concern_t *rc = NULL;
@@ -574,6 +621,7 @@ session_opts_new (bson_t *bson, bson_error_t *error)
 
    bp = bson_parser_new ();
    bson_parser_bool_optional (bp, "causalConsistency", &causal_consistency);
+   bson_parser_bool_optional (bp, "snapshot", &snapshot);
    bson_parser_doc_optional (
       bp, "defaultTransactionOptions", &default_transaction_opts);
    if (!bson_parser_parse (bp, bson, error)) {
@@ -584,6 +632,9 @@ session_opts_new (bson_t *bson, bson_error_t *error)
    if (causal_consistency) {
       mongoc_session_opts_set_causal_consistency (opts, *causal_consistency);
    }
+   if (snapshot) {
+      mongoc_session_opts_set_snapshot (opts, *snapshot);
+   }
 
    if (default_transaction_opts) {
       bp_opts = bson_parser_new ();
@@ -868,6 +919,11 @@ entity_destroy (entity_t *entity)
       mongoc_gridfs_bucket_t *bucket = entity->value;
 
       mongoc_gridfs_bucket_destroy (bucket);
+   } else if (0 == strcmp ("findcursor", entity->type)) {
+      entity_findcursor_t *findcursor = entity->value;
+
+      mongoc_cursor_destroy (findcursor->cursor);
+      bson_free (findcursor);
    } else {
       test_error ("Attempting to destroy unrecognized entity type: %s, id: %s",
                   entity->type,
@@ -903,6 +959,20 @@ entity_map_get (entity_map_t *entity_map, const char *id, bson_error_t *error)
    return NULL;
 }
 
+bool
+entity_map_delete (entity_map_t *em, const char *id, bson_error_t *error)
+{
+   entity_t *entity = entity_map_get (em, id, error);
+   if (!entity) {
+      return false;
+   }
+
+   LL_DELETE (em->entities, entity);
+   entity_destroy (entity);
+
+   return true;
+}
+
 static entity_t *
 _entity_map_get_by_type (entity_map_t *entity_map,
                          const char *id,
@@ -977,6 +1047,19 @@ entity_map_get_changestream (entity_map_t *entity_map,
    return (mongoc_change_stream_t *) entity->value;
 }
 
+entity_findcursor_t *
+entity_map_get_findcursor (entity_map_t *entity_map,
+                           const char *id,
+                           bson_error_t *error)
+{
+   entity_t *entity =
+      _entity_map_get_by_type (entity_map, id, "findcursor", error);
+   if (!entity) {
+      return NULL;
+   }
+   return (entity_findcursor_t *) entity->value;
+}
+
 bson_val_t *
 entity_map_get_bson (entity_map_t *entity_map,
                      const char *id,
@@ -1072,6 +1155,45 @@ entity_map_add_changestream (entity_map_t *em,
       em, id, "changestream", (void *) changestream, error);
 }
 
+void
+entity_findcursor_iterate_until_document_or_error (
+   entity_findcursor_t *findcursor,
+   const bson_t **document,
+   bson_error_t *error,
+   const bson_t **error_document)
+{
+   *document = NULL;
+
+   if (findcursor->first_result) {
+      *document = findcursor->first_result;
+      findcursor->first_result = NULL;
+      return;
+   }
+
+   while (!mongoc_cursor_next (findcursor->cursor, document)) {
+      if (mongoc_cursor_error_document (
+             findcursor->cursor, error, error_document)) {
+         return;
+      }
+   }
+}
+
+bool
+entity_map_add_findcursor (entity_map_t *em,
+                           const char *id,
+                           mongoc_cursor_t *cursor,
+                           const bson_t *first_result,
+                           bson_error_t *error)
+{
+   entity_findcursor_t *findcursor;
+
+   findcursor =
+      (entity_findcursor_t *) bson_malloc0 (sizeof (entity_findcursor_t));
+   findcursor->cursor = cursor;
+   findcursor->first_result = first_result;
+   return _entity_map_add (em, id, "findcursor", (void *) findcursor, error);
+}
+
 bool
 entity_map_add_bson (entity_map_t *em,
                      const char *id,
